@@ -1578,6 +1578,10 @@ scanner(Complist q)
 	zerr("no idea how you got this error message.", NULL, 0);
 }
 
+/* Flags passed down to guts when compiling */
+#define GF_PATHADD	1	/* file glob, adding path components */
+#define GF_TOPLEV	2	/* outside (), so ~ ends main match */
+
 static char *pptr;		/* current place in string being matched */
 static Comp tail = 0;
 static int first;		/* are leading dots special? */
@@ -1843,7 +1847,7 @@ parsereg(char *str)
     remnulargs(str);
     mode = 1;			/* no path components */
     pptr = str;
-    return parsecompsw(0);
+    return parsecompsw(GF_TOPLEV);
 }
 
 /* Parse a series of path components pointed to by pptr */
@@ -1905,7 +1909,7 @@ parsecomplist(void)
 	}
     } else {
 	/* parse single path component */
-	if (!(c1 = parsecompsw(1)))
+	if (!(c1 = parsecompsw(GF_PATHADD|GF_TOPLEV)))
 	    return NULL;
 	/* then do the remaining path compoents */
 	if (*pptr == '/' || !*pptr) {
@@ -1926,7 +1930,7 @@ parsecomplist(void)
 
 /**/
 Comp
-parsecomp(void)
+parsecomp(int gflag)
 {
     Comp c = (Comp) alloc(sizeof *c), c1, c2;
     char *cstr, *ls = NULL;
@@ -1946,7 +1950,7 @@ parsecomp(void)
 	    /* negate remaining pattern */
 	    pptr++;
 	    c->str = dupstrpfx(cstr, pptr - cstr);
-	    if (!(c->next = parsecomp()))
+	    if (!(c->next = parsecomp(gflag)))
 		return NULL;
 	    return c;
 	}
@@ -1963,7 +1967,7 @@ parsecomp(void)
 	    c1 = (Comp) alloc(sizeof *c1);
 	    *(c1->str = dupstring("?")) = Quest;
 	    c1->stat |= C_ONEHASH;
-	    if (!(c2 = parsecomp()))
+	    if (!(c2 = parsecomp(gflag)))
 		return NULL;
 	    c1->next = c2;
 	    c->next = c1;
@@ -1990,7 +1994,7 @@ parsecomp(void)
 		}
 	    }
 	    /* Parse the remaining pattern following the group... */
-	    if (!(c1 = parsecomp()))
+	    if (!(c1 = parsecomp(gflag)))
 		return NULL;
 	    /* ...remembering what comes after it... */
 	    tail = dpnd ? NULL : c1;
@@ -2027,7 +2031,7 @@ parsecomp(void)
 	    c1->next = c2;
 	    c2->stat |= C_ONEHASH;
 	    /* parse the rest of the pattern and return. */
-	    c2->next = parsecomp();
+	    c2->next = parsecomp(gflag);
 	    if (!c2->next)
 		return NULL;
 	    c->str = dupstrpfx(cstr, ls - cstr);
@@ -2059,7 +2063,8 @@ parsecomp(void)
 	pptr++;
     }
     /* mark if last pattern component in path component or pattern */
-    if (*pptr == '/' || !*pptr)
+    if (*pptr == '/' || !*pptr ||
+	(isset(EXTENDEDGLOB) && *pptr == Tilde && (gflag & GF_TOPLEV)))
 	c->stat |= C_LAST;
     c->str = dupstrpfx(cstr, pptr - cstr);
     return c;
@@ -2069,11 +2074,11 @@ parsecomp(void)
 
 /**/
 Comp
-parsecompsw(int pathadd)
+parsecompsw(int gflag)
 {
     Comp c1, c2, c3, excl = NULL;
 
-    c1 = parsecomp();
+    c1 = parsecomp(gflag);
     if (!c1)
 	return NULL;
     if (isset(EXTENDEDGLOB) && *pptr == Tilde) {
@@ -2082,7 +2087,7 @@ parsecompsw(int pathadd)
 
 	mode = 1;
 	pptr++;
-	excl = parsecomp();
+	excl = parsecomp(gflag);
 	mode = oldmode;
 	if (!excl)
 	    return NULL;
@@ -2093,7 +2098,7 @@ parsecompsw(int pathadd)
 	if (*pptr == Bar) {
 	    /* get the next alternative after the | */
 	    pptr++;
-	    c3 = parsecompsw(pathadd);
+	    c3 = parsecompsw(gflag);
 	    if (!c3)
 		return NULL;
 	} else {
@@ -2106,7 +2111,7 @@ parsecompsw(int pathadd)
 	c2->left = c1;
 	c2->right = c3;
 	c2->exclude = excl;
-	if (pathadd)
+	if (gflag & GF_PATHADD)
 	    c2->stat |= C_PATHADD;
 	return c2;
     }
