@@ -137,8 +137,9 @@ update_job(Job jn)
     job = jn - jobtab;   /* compute job number */
 
     if (somestopped) {
-	if (jn->stty_in_env && !jn->ty) {
-	    jn->ty = (struct ttyinfo *) zalloc(sizeof(struct ttyinfo));
+	if (shout && job == thisjob) {
+	    if (!jn->ty)
+		jn->ty = (struct ttyinfo *) zalloc(sizeof(struct ttyinfo));
 	    gettyinfo(jn->ty);
 	}
 	if (jn->stat & STAT_STOPPED)
@@ -154,7 +155,8 @@ update_job(Job jn)
 	}
     }
 
-    if (shout && !ttyfrozen && !jn->stty_in_env)
+    if (shout && !ttyfrozen && !jn->stty_in_env &&
+	job == thisjob && !somestopped)
 	gettyinfo(&shttyinfo);
 
     if (isset(MONITOR)) {
