@@ -99,13 +99,11 @@ createparamtable(void)
 	old_environ = environ;
 	num_env = arrlen(old_environ);
 	environ = (char **) zalloc(sizeof(char *) * (num_env + 1));
-	for (envp = environ, envp2 = old_environ; *envp2; envp2++)
-	    *envp++ = ztrdup(*envp2);
-	*envp = NULL;
+	*environ = NULL;
 
 	/* Now incorporate environment variables we are inheriting *
 	 * into the parameter hash table.                          */
-	for (envp = environ, envp2 = old_environ; *envp2; envp++, envp2++) {
+	for (envp = environ, envp2 = old_environ; *envp2; envp2++) {
 	    for (str = *envp2; *str && *str != '='; str++);
 	    if (*str == '=') {
 		iname = NULL;
@@ -114,13 +112,15 @@ createparamtable(void)
 		    iname = *envp2;
 		    if ((!(pm = (Param) paramtab->getnode(paramtab, iname)) ||
 			 !(pm->flags & PM_DONTIMPORT)) &&
-			(pm = setsparam(iname, metafy(str + 1, -1, META_DUP)))) {
+			(pm = setsparam(iname, metafy(str + 1, -1, META_DUP))) &&
+			!(pm->flags & PM_EXPORTED)) {
+			*str = '=';
 			pm->flags |= PM_EXPORTED;
-			pm->env = *envp;
+			pm->env = *envp++ = ztrdup(*envp2);
+			*envp = NULL;
 			if (pm->flags & PM_SPECIAL)
-			    pm->env = replenv(pm->env, getsparam(iname));
-		    } else if (pm)
-			for (t = envp--; (t[0] = t[1]); t++);
+			    pm->env = replenv(pm->env, getsparam(pm->nam));
+		    }
 		}
 		*str = '=';
 	    }
