@@ -1459,10 +1459,11 @@ paramsubst(LinkList l, LinkNode n, char **str, int qt, int ssub)
 		    return NULL;
 		if (qt && !*x && isarr != 2)
 		    y = dupstring(nulstring);
-		else if (globsubst)
-		    tokenize(y = dupstring(x));
-		else
-		    y = x;
+		else {
+		    y = dupstring(x);
+		    if (globsubst)
+			tokenize(y);
+		}
 		insertlinknode(l, n, (void *) y), incnode(n);
 	    }
 
