@@ -758,6 +758,8 @@ refreshline(int ln)
 void
 moveto(int ln, int cl)
 {
+    int c;
+
     if (vcs == winw) {
 	vln++, vcs = 0;
 	if (!hasam) {
@@ -765,12 +767,13 @@ moveto(int ln, int cl)
 	    zputc('\n', shout);
 	} else {
 	    if ((vln < nlnct) && nbuf[vln] && *nbuf[vln])
-		zputc(*nbuf[vln], shout);
+		c = *nbuf[vln];
 	    else
-		zputc(' ', shout);
+		c = ' ';
+	    zputc(c, shout);
 	    zputc('\r', shout);
 	    if ((vln < olnct) && obuf[vln] && *obuf[vln])
-		*obuf[vln] = *nbuf[vln];
+		*obuf[vln] = c;
 	}
     }
 
