@@ -197,8 +197,7 @@ enum {
 #define INP_ALIAS     (1<<1)	/* expanding alias or history              */
 #define INP_HIST      (1<<2)	/* expanding history                       */
 #define INP_CONT      (1<<3)	/* continue onto previously stacked input  */
-#define INP_ALMORE    (1<<4)	/* alias ended with space, continue expn.  */
-#define INP_ALCONT    (1<<5)	/* stack is continued from alias expn.     */
+#define INP_ALCONT    (1<<4)	/* stack is continued from alias expn.     */
 
 /* Flags for metafy */
 #define META_REALLOC	0
@@ -1187,13 +1186,29 @@ struct ttyinfo {
 #endif
 };
 
-/* flags for termok state: usually the test for whether termok   *
- * is set is enough: in the refresh code we need to know whether *
- * the terminal has an up capability.                            */
+/* defines for whether tabs expand to spaces */
+#if defined(HAVE_TERMIOS_H) || defined(HAVE_TERMIO_H)
+#define SGTTYFLAG       shttyinfo.tio.c_oflag
+#else   /* we're using sgtty */
+#define SGTTYFLAG       shttyinfo.sgttyb.sg_flags
+#endif
+# ifdef TAB3
+#define SGTABTYPE       TAB3
+# else
+#  ifdef OXTABS
+#define SGTABTYPE       OXTABS
+#  else
+#define SGTABTYPE       XTABS
+#  endif
+# endif
 
-#define TERM_BAD	0
-#define TERM_OK		1
-#define TERM_NOUP	2
+/* flags for termflags */
+
+#define TERM_BAD	0x01	/* terminal has extremely basic capabilities */
+#define TERM_UNKNOWN	0x02	/* unknown terminal type */
+#define TERM_NOUP	0x04	/* terminal has no up capability */
+#define TERM_SHORT	0x08	/* terminal is < 3 lines high */
+#define TERM_NARROW	0x10	/* terminal is < 3 columns wide */
 
 /* interesting termcap strings */
 
